package com.example.afrishop_v3.quartz;

import com.example.afrishop_v3.enums.CategoryEnum;
import com.example.afrishop_v3.models.*;
import com.example.afrishop_v3.repository.TbCfDescripitonRepository;
import com.example.afrishop_v3.repository.TbCfGoodstwotypeRepository;
import com.example.afrishop_v3.repository.TbCfGoodstypeRepository;
import com.example.afrishop_v3.repository.TbCfStationItemRepository;
import com.example.afrishop_v3.util.HttpClientUtil;
import com.example.afrishop_v3.util.IdUtil;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.URISyntaxException;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

@Component
@Slf4j
public class ItemTask implements InitializingBean, DisposableBean {



    @Autowired
    private TbCfGoodstypeRepository tbCfGoodstypeRepository;
    @Autowired
    private TbCfGoodstwotypeRepository goodstwotypeRepository;
    @Autowired
    private TbCfDescripitonRepository descripitonRepository;
    @Autowired
    private TbCfStationItemRepository tbCfStationItemRepository;

    private volatile int sort=0;
    private volatile Integer page=1;
    private final Integer limit=48;
    private int total_page=0;

    private final String CATEGORY_URL="https://apiv2.lovelywholesale.com/lw-api/head/switch-top-category";
    private final String url1="https://apiv2.lovelywholesale.com/lw-api/list/get_child_cat";
    private final String ITEM_URL="https://goapi.lovelywholesale.com/search/category/products";
    private final String ITEM_DETAIL_URL="https://apiv2.lovelywholesale.com/lw-api/product/item/";


    private final int num=Runtime.getRuntime().availableProcessors();
    private final ThreadPoolExecutor executor=new ThreadPoolExecutor(num,20,10, TimeUnit.SECONDS, new LinkedBlockingQueue(1024), Executors.defaultThreadFactory(),new ThreadPoolExecutor.DiscardPolicy());
    private final ReentrantLock lock=new ReentrantLock();


    @Override
    public void afterPropertiesSet() throws Exception {
        importData();
    }

    @Override
    public void destroy() throws Exception {
        executor.shutdown();
    }

    //=================lovelywholesale商品导入逻辑=================
    //棒棒哒！思密哒！
    //TODO 温馨提示：是要补充的部分！

    /**
     * 商品主体信息
     */
    public void importData() {
        for (int i = 0; i < num; i++) {
            executor.execute(()-> {
                log.info("线程"+Thread.currentThread().getName()+"开始执行任务！");
                try {
                    importCategory();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });
        }
    }


    /**
     * 商品详情、sku、属性。。。(还未做)
     */
    public void importDetail(){
        for (int i = 0; i < 16; i++) {
            executor.execute(()->{
                List<TbCfStationItem> items = tbCfStationItemRepository.findAllByTempIdIsNotNull();
                items.forEach(item->{
                    String goods_id = item.getTempId();
                    try {
                        String result = HttpClientUtil.createConnection(ITEM_DETAIL_URL+goods_id, null, "utf-8");
                        //TODO 待处理。。。
                        //商品详情、属性、sku等，处理result，入库如下几张表
                        // tb_cf_item_skus tb_cf_item_param tb_cf_item_desc attributes attributes_desc tb_cf_category tb_cf_option
                        // tb_cf_item_comment(商品评价：可选)
                        // 难受。。。
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                });
            });
        }

    }

    /**
     * 一、二、三级分类导入
     * @throws IOException
     */
//    @Test
    public void importCategory() throws IOException {
        String topCategory = null;
        List<Integer> ids= Arrays.asList(1,3,5,4);
        for (int i = 0; i < ids.size(); i++) {

            Integer catId = ids.get(i);
            if (CategoryEnum.WOMEN.getCategoryId()==catId){
                topCategory=CategoryEnum.WOMEN.getCategoryName();
            }else if (CategoryEnum.CURVE.getCategoryId()==catId){
                topCategory=CategoryEnum.CURVE.getCategoryName();
            }else if (CategoryEnum.MEN.getCategoryId()==catId){
                topCategory=CategoryEnum.MEN.getCategoryName();
            }else if (CategoryEnum.KIDS.getCategoryId()==catId){
                topCategory=CategoryEnum.KIDS.getCategoryName();
            }

//            lock.lock();
            String categoryId= IdUtil.createIdbyUUID();
            //一级分类
            categoryId = importTopCategory(topCategory,categoryId,catId);
            Map<String,Object> params=new LinkedHashMap<>();
            params.put("top_cat_id",catId);
            String response=HttpClientUtil.sendPostWithBodyParameter(CATEGORY_URL,params);
            JSONObject jsonObject = JSONObject.fromObject(response);
            JSONObject data = jsonObject.getJSONObject("data");
            JSONArray categorys = data.getJSONArray("categorys");
            for (int j = 0; j < categorys.size(); j++) {
                String categoryId2=IdUtil.createIdbyUUID();
                JSONObject category = categorys.getJSONObject(j);
                String tempId2 = category.getString("nav_id");
                String navName = category.getString("nav_name");

                //二级分类
                TbCfGoodstwotype goodstwotype=new TbCfGoodstwotype();
                if (!goodstwotypeRepository.existsByGoodstwotypeTitleAndTempId(navName,tempId2)){
                    goodstwotype.setGoodstwotypeId(categoryId2);
                    goodstwotype.setGoodstypeId(categoryId);
                    goodstwotype.setGoodstwotypeTitle(navName);
                }else {
                    goodstwotype = goodstwotypeRepository.findFirstByGoodstwotypeTitleAndTempId(navName,tempId2);
                    categoryId2=goodstwotype.getGoodstwotypeId();

                }

                goodstwotype.setTempId(tempId2);
                goodstwotypeRepository.save(goodstwotype);

                //三级分类
                Map<String, String> children = getCategoryChildren(tempId2);
                String finalCategoryId = categoryId;
                String finalCategoryId1 = categoryId2;
                children.entrySet().forEach(c->{
                    String id = c.getKey();
                    String[] split = c.getValue().split("-");
                    String tempId3 =split[0];
                    String name =split[1];
                    TbCfDescripiton descripiton=new TbCfDescripiton();
                    String categoryId3=IdUtil.createIdbyUUID();
                    if (!descripitonRepository.existsByDescripitionNameAndTempId(name,tempId3)){
                        descripiton.setDescripitionId(categoryId3);
                        descripiton.setDescripitionName(name);
                        descripiton.setGoodstypeId(finalCategoryId);
                        descripiton.setGoodstwotypeId(finalCategoryId1);

                    }else {
                        descripiton  = descripitonRepository.findFirstByDescripitionNameAndTempId(name,tempId3);
                        categoryId3=descripiton.getDescripitionId();
                    }
                    descripiton.setTempId(tempId3);
                    descripitonRepository.save(descripiton);
                    String str=catId+","+tempId2+","+tempId3;
                    //商品数据
                    try {
                        importItems(finalCategoryId, finalCategoryId1,categoryId3,id,str);
//                        lock.unlock();
                    } catch (IOException | URISyntaxException e) {
//                        lock.unlock();
                        e.printStackTrace();
                    }
                });
            }
        }

    }

    /**
     * 商品导入逻辑
     * @param categoryId
     * @param categoryId2
     * @param categoryId3
     * @throws IOException
     */
    public void importItems(String categoryId,String categoryId2,String categoryId3,String catId,String str) throws IOException, URISyntaxException {
        do {
            page++ ;
            Map<String,Object> params=new LinkedHashMap<>();
            params.put("offset",page);
            params.put("limit",limit);
            params.put("category_id",catId);
            params.put("original_category_id",catId);
            params.put("is_virtual",1);
            String response = HttpClientUtil.sendPostWithBodyParameter(ITEM_URL, params);
            JSONObject jsonObject = JSONObject.fromObject(response);
            JSONObject data = jsonObject.getJSONObject("data");
            total_page = data.getInt("total_page");
            JSONArray items = data.getJSONArray("search_list");
            for (int i = 0; i < items.size(); i++) {

                JSONObject goods = items.getJSONObject(i);
                String goods_id = goods.getString("goods_id");
                String goods_name = goods.getString("goods_name");
                //现价
                String usd_shop_price = goods.getString("usd_shop_price");
                //原价
                String usd_market_price = goods.getString("usd_market_price");
                String aws_goods_thumb = goods.getString("aws_goods_thumb");
                //TODO
                //看情况改吧
                //商品入库
                TbCfStationItem item=new TbCfStationItem();
                item.setItemId(IdUtil.createIdbyUUID());
                item.setTempId(goods_id);
                item.setItemName(goods_name);
                item.setItemBrief(goods_name);
                item.setEnableFlag(2);
                item.setCreateTime(new Date());
                item.setItemCode("2021-"+new Random().nextInt(999999));
                item.setDiscountPrice(new BigDecimal(usd_shop_price));
                item.setItemPrice(new BigDecimal(usd_market_price));
                item.setExpress(new TbCfExpressTemplate("ef57cb80e2e74a42850119c014b06c96"));
                //处理标记
                item.setHandled("W");
                item.setItemCategory(categoryId);
                item.setItemCategorytwo(categoryId2);
                item.setItemDescritionId(categoryId3);
                //商品库存
                item.setItemCount(500L);
                item.setItemTop(0);
                item.setItemImg(aws_goods_thumb);
                //额外的属性，看情况填充
                item.setItemTags(str);//搜索关键字
                item.setSupplier(null);//供应商
                tbCfStationItemRepository.save(item);
            }

        }while (page<total_page);


    }

    /**
     * 一级分类导入
     */
    public String importTopCategory(String topCategory,String categoryId,int catId){

        TbCfGoodstype goodstype=new TbCfGoodstype();
        if (!tbCfGoodstypeRepository.existsByGoodstypeTitle(topCategory)){
            //新增
            goodstype.setGoodstypeId(categoryId);
            goodstype.setGoodstypeTitle(topCategory);
            goodstype.setTempId(catId);
            tbCfGoodstypeRepository.save(goodstype);
            return categoryId;
        }else {
            TbCfGoodstype type = tbCfGoodstypeRepository.findByGoodstypeTitle(topCategory);
            type.setTempId(catId);
            tbCfGoodstypeRepository.save(type);
            return type.getGoodstypeId();
        }
    }


    /**
     * 通过二级分类id查询三级分类
     * @param categoryId
     * @return
     * @throws IOException
     */
    public Map<String,String> getCategoryChildren(String categoryId) throws IOException {
        Map<String,String> results=new LinkedHashMap<>();
        Map<String,Object> params=new LinkedHashMap<>();
        params.put("nav_id",categoryId);
        String response=HttpClientUtil.sendPostWithBodyParameter(url1,params);
        JSONObject jsonObject = JSONObject.fromObject(response);
        JSONObject data = jsonObject.getJSONObject("data");
        JSONArray categorys = data.getJSONArray("category");
        for (int j = 0; j < categorys.size(); j++) {
            JSONObject category = categorys.getJSONObject(j);
            JSONArray childs = category.getJSONArray("child");
            for (int i = 0; i < childs.size(); i++) {
                JSONObject child = childs.getJSONObject(i);
                String nav_id = child.getString("nav_id");
                String name = child.getString("nav_name");
                String cat_id = child.getJSONObject("cat_info").getString("cat_id");
                results.put(cat_id,nav_id+"-"+name);
            }
        }
        return results;
    }

}
